@@ -4,7 +4,9 @@ rbind.tbl_ts <- function(...) {
   data <- list2(...)[[1]]
   x <- dplyr::bind_rows(...)
   x <- retain_tsibble(x, key(data), index(data))
-  update_meta(x, data, ordered = NULL)
+  int_int <- map_lgl(list2(...), ~ !is_empty(. %@% "interval"))
+  int <- purrr::reduce(int_int, `||`)
+  update_meta(x, data, ordered = NULL, interval = int)
 }
 
 #' @method cbind tbl_ts
